import os
import sys
import unittest
import StringIO
from geneagrapher import geneagrapher
from geneagrapher.cache_grabber import CacheGrabber
from geneagrapher.graph import Graph
from local_data_grabber import LocalDataGrabber


class TestGeneagrapherMethods(unittest.TestCase):
    """Unit tests for the geneagrapher.Geneagrapher class."""
    def setUp(self):
        self.ggrapher = geneagrapher.Geneagrapher()

    def test_init(self):
        """Test constructor."""
        self.assertEqual(isinstance(self.ggrapher.graph, Graph), True)
        self.assertEqual(self.ggrapher.seed_ids, [])
        self.assertEqual(self.ggrapher.get_ancestors, False)
        self.assertEqual(self.ggrapher.get_descendants, False)
        self.assertEqual(self.ggrapher.verbose, False)
        self.assertEqual(self.ggrapher.write_filename, None)
        self.assertEqual(self.ggrapher.use_cache, True)
        self.assertEqual(self.ggrapher.cache_file, 'geneacache')

    def test_parse_empty(self):
        """Test parse_input() with no arguments."""
        sys.argv = ['geneagrapher']

        # Redirect stderr to capture output.
        stderr = sys.stderr
        stderr_intercept = StringIO.StringIO()
        sys.stderr = stderr_intercept

        expected = """usage: geneagrapher [-h] [--version] [-f FILE] [-a] \
[-d] [--disable-cache]
                    [--cache-file FILE] [-v]
                    ID [ID ...]
geneagrapher: error: too few arguments
"""
        try:
            self.ggrapher.parse_input()
        except SystemExit:
            self.assertEqual(stderr_intercept.getvalue().decode('utf-8'),
                             expected)

        sys.stderr = stderr

    def test_parse_default(self):
        """Test parse_input() with no options."""
        sys.argv = ['geneagrapher', '3']
        self.ggrapher.get_ancestors = False
        self.ggrapher.get_descendents = True
        self.ggrapher.write_filename = "filler"
        self.ggrapher.parse_input()
        self.assertEqual(self.ggrapher.get_ancestors, False)
        self.assertEqual(self.ggrapher.get_descendants, False)
        self.assertEqual(self.ggrapher.verbose, False)
        self.assertEqual(self.ggrapher.write_filename, None)
        self.assertEqual(self.ggrapher.use_cache, True)
        self.assertEqual(self.ggrapher.cache_file, 'geneacache')
        self.assertEqual(self.ggrapher.seed_ids, [3])

    def test_parse_options(self):
        """Test parse_input() with options."""
        sys.argv = ['geneagrapher', '--with-ancestors', '--with-descendants',
                    '--file=filler', '--verbose', '--disable-cache',
                    '--cache-file', 'foo', '3', '43']
        self.ggrapher.parse_input()
        self.assertEqual(self.ggrapher.get_ancestors, True)
        self.assertEqual(self.ggrapher.get_descendants, True)
        self.assertEqual(self.ggrapher.verbose, True)
        self.assertEqual(self.ggrapher.write_filename, "filler")
        self.assertEqual(self.ggrapher.use_cache, False)
        self.assertEqual(self.ggrapher.cache_file, 'foo')
        self.assertEqual(self.ggrapher.seed_ids, [3, 43])

    def test_parse_short_options(self):
        """Test parse_input() with short versions of the options."""
        sys.argv = ['geneagrapher', '-a', '-d', '-f', 'filler', '-v', '3',
                    '43']
        self.ggrapher.parse_input()
        self.assertEqual(self.ggrapher.get_ancestors, True)
        self.assertEqual(self.ggrapher.get_descendants, True)
        self.assertEqual(self.ggrapher.verbose, True)
        self.assertEqual(self.ggrapher.write_filename, "filler")
        self.assertEqual(self.ggrapher.use_cache, True)
        self.assertEqual(self.ggrapher.cache_file, 'geneacache')
        self.assertEqual(self.ggrapher.seed_ids, [3, 43])

    def test_build_graph_complete_only_self(self):
        """Graph building with no ancestors or descendants."""
        self.ggrapher.seed_ids.append(127946)
        self.ggrapher.build_graph_complete(LocalDataGrabber)
        graph = self.ggrapher.graph
        self.assertEqual(len(graph), 1)
        self.assertTrue(127946 in graph)

        node = graph[127946]
        self.assertEqual(node.ancestors, set())
        self.assertEqual(node.descendants, set())

        record = node.record
        self.assertEqual(record.name, "Christian   Thomasius")
        self.assertEqual(record.institution, None)
        self.assertEqual(record.year, 1672)
        self.assertEqual(record.id, 127946)

    def test_build_graph_complete_only_self_verbose_cache_grabber(self):
        """Graph building with no ancestors or descendants using the cache
        grabber to verify its verbose printing."""
        self.ggrapher.verbose = True
        self.ggrapher.seed_ids.append(127946)

        # Redirect stdout to capture output.
        stdout = sys.stdout
        stdout_intercept = StringIO.StringIO()
        sys.stdout = stdout_intercept
        cache_fname = LocalDataGrabber.data_file(
            'geneagrapher_verbose_cache_grabber_test')
        expiration = 1e15
        self.ggrapher.build_graph_complete(CacheGrabber, filename=cache_fname,
                                           expiration_interval=expiration)
        sys.stdout = stdout

        self.assertEqual(stdout_intercept.getvalue().decode('utf-8'),
                         u"Grabbing record #127946...cache hit\n")

    def test_build_graph_complete_only_self_verbose_error(self):
        """Graph building with no ancestors or descendants given a bad
        ID."""
        self.ggrapher.verbose = True
        self.ggrapher.seed_ids.append(999999999)

        self.assertRaises(ValueError, self.ggrapher.build_graph_complete,
                          LocalDataGrabber)

    def test_build_graph_complete_only_self_verbose_error_long(self):
        """Graph building with no ancestors or descendants given a bad
        ID of the long variety."""
        self.ggrapher.verbose = True
        self.ggrapher.seed_ids.append(999999999999999999999)

        self.assertRaises(ValueError, self.ggrapher.build_graph_complete,
                          LocalDataGrabber)

    def test_build_graph_complete_only_self_verbose(self):
        """Graph building with no ancestors or descendants."""
        self.ggrapher.verbose = True
        self.ggrapher.seed_ids.append(127946)

        # Redirect stdout to capture output.
        stdout = sys.stdout
        stdout_intercept = StringIO.StringIO()
        sys.stdout = stdout_intercept
        self.ggrapher.build_graph_complete(LocalDataGrabber)
        sys.stdout = stdout

        graph = self.ggrapher.graph
        self.assertEqual(len(graph), 1)
        self.assertTrue(127946 in graph)

        node = graph[127946]
        self.assertEqual(node.ancestors, set())
        self.assertEqual(node.descendants, set())

        record = node.record
        self.assertEqual(record.name, "Christian   Thomasius")
        self.assertEqual(record.institution, None)
        self.assertEqual(record.year, 1672)
        self.assertEqual(record.id, 127946)

        self.assertEqual(stdout_intercept.getvalue().decode('utf-8'),
                         u"Grabbing record #127946...\n")

    def test_build_graph_complete_with_ancestors(self):
        """Graph building with ancestors."""
        self.ggrapher.seed_ids.append(127946)
        self.ggrapher.get_ancestors = True
        self.ggrapher.build_graph_complete(LocalDataGrabber)
        graph = self.ggrapher.graph
        self.assertEqual(len(graph), 4)
        self.assertTrue(127946 in graph)
        self.assertTrue(137717 in graph)
        self.assertTrue(137705 in graph)
        self.assertTrue(143630 in graph)

        node = graph[127946]
        self.assertEqual(node.ancestors, set([137717, 137705]))
        self.assertEqual(node.descendants, set())

        record = node.record
        self.assertEqual(record.name, "Christian   Thomasius")
        self.assertEqual(record.institution, None)
        self.assertEqual(record.year, 1672)
        self.assertEqual(record.id, 127946)

        node = graph[137717]
        self.assertEqual(node.ancestors, set([]))
        self.assertEqual(node.descendants, set([127946]))

        record = node.record
        self.assertEqual(record.name, "Valentin  Alberti")
        self.assertEqual(record.institution, u"Universit\xe4t Leipzig")
        self.assertEqual(record.year, 1678)
        self.assertEqual(record.id, 137717)

        node = graph[137705]
        self.assertEqual(node.ancestors, set([143630]))
        self.assertEqual(node.descendants, set([127946]))

        record = node.record
        self.assertEqual(record.name, "Jakob  Thomasius")
        self.assertEqual(record.institution, u"Universit\xe4t Leipzig")
        self.assertEqual(record.year, 1643)
        self.assertEqual(record.id, 137705)

        node = graph[143630]
        self.assertEqual(node.ancestors, set([]))
        self.assertEqual(node.descendants, set([137705]))

        record = node.record
        self.assertEqual(record.name, "Friedrich  Leibniz")
        self.assertEqual(record.institution, None)
        self.assertEqual(record.year, None)
        self.assertEqual(record.id, 143630)

    def test_build_graph_complete_with_descendants(self):
        """Graph building with descendants."""
        self.ggrapher.seed_ids.append(79568)
        self.ggrapher.get_descendants = True
        self.ggrapher.build_graph_complete(LocalDataGrabber)
        graph = self.ggrapher.graph
        self.assertEqual(len(graph), 3)
        self.assertTrue(79568 in graph)
        self.assertTrue(79562 in graph)
        self.assertTrue(99457 in graph)

        node = graph[79568]
        self.assertEqual(node.ancestors, set())
        self.assertEqual(node.descendants, set([79562, 99457]))

        record = node.record
        self.assertEqual(record.name, "Ramdas  Kumaresan")
        self.assertEqual(record.institution, u"University of Rhode Island")
        self.assertEqual(record.year, 1982)
        self.assertEqual(record.id, 79568)

        node = graph[79562]
        self.assertEqual(node.ancestors, set([79568]))
        self.assertEqual(node.descendants, set([]))

        record = node.record
        self.assertEqual(record.name, "C. S. Ramalingam")
        self.assertEqual(record.institution, u"University of Rhode Island")
        self.assertEqual(record.year, 1995)
        self.assertEqual(record.id, 79562)

        node = graph[99457]
        self.assertEqual(node.ancestors, set([79568]))
        self.assertEqual(node.descendants, set([]))

        record = node.record
        self.assertEqual(record.name, "Yadong  Wang")
        self.assertEqual(record.institution, u"University of Rhode Island")
        self.assertEqual(record.year, 2003)
        self.assertEqual(record.id, 99457)

    def test_build_graph_complete_bad_id(self):
        """Graph building with a bad ID."""
        self.ggrapher.seed_ids.append(79568583832)
        self.assertRaises(ValueError, self.ggrapher.build_graph_complete,
                          LocalDataGrabber)

        try:
            self.ggrapher.build_graph_complete(LocalDataGrabber)
        except ValueError as e:
            self.assertEqual(str(e), "Invalid id 79568583832")
        else:
            self.fail()

    def test_end_to_end_self_stdout(self):
        """Complete test getting no ancestors or descendants and writing the
        result to stdout."""
        sys.argv = ['geneagrapher', '30484']
        self.ggrapher.parse_input()
        self.assertEqual(self.ggrapher.get_ancestors, False)
        self.assertEqual(self.ggrapher.get_descendants, False)
        self.assertEqual(self.ggrapher.verbose, False)
        self.assertEqual(self.ggrapher.write_filename, None)
        self.assertEqual(self.ggrapher.seed_ids, [30484])

        self.ggrapher.build_graph_complete(LocalDataGrabber)

        # Redirect stdout to capture output.
        stdout = sys.stdout
        stdout_intercept = StringIO.StringIO()
        sys.stdout = stdout_intercept
        self.ggrapher.generate_dot_file()
        sys.stdout = stdout

        expected = u"""digraph genealogy {
    graph [charset="utf-8"];
    node [shape=plaintext];
    edge [style=bold];

    30484 [label="Peter Chris Pappas \\nThe Pennsylvania State University \
(1982)"];

}
"""
        self.assertEqual(stdout_intercept.getvalue().decode('utf-8'), expected)

    def test_end_to_end_cached_self_stdout(self):
        """Complete test using cache getting no ancestors or descendants and
        writing the result to stdout."""
        cache_fname = LocalDataGrabber.data_file(
            'end-to-end-30484')
        sys.argv = ['geneagrapher', '--cache-file', cache_fname, '30484']
        self.ggrapher.parse_input()
        self.assertEqual(self.ggrapher.get_ancestors, False)
        self.assertEqual(self.ggrapher.get_descendants, False)
        self.assertEqual(self.ggrapher.verbose, False)
        self.assertEqual(self.ggrapher.write_filename, None)
        self.assertEqual(self.ggrapher.seed_ids, [30484])

        self.ggrapher.build_graph_complete(CacheGrabber,
                                           filename=self.ggrapher.cache_file)

        # Redirect stdout to capture output.
        stdout = sys.stdout
        stdout_intercept = StringIO.StringIO()
        sys.stdout = stdout_intercept
        self.ggrapher.generate_dot_file()
        sys.stdout = stdout

        expected = u"""digraph genealogy {
    graph [charset="utf-8"];
    node [shape=plaintext];
    edge [style=bold];

    30484 [label="Peter Chris Pappas \\nThe Pennsylvania State University \
(1982)"];

}
"""
        self.assertEqual(stdout_intercept.getvalue().decode('utf-8'), expected)

    def test_end_to_end_ancestors_stdout(self):
        """
        Complete test getting with ancestors, writing the result to stdout.
        """
        sys.argv = ['geneagrapher', '-a', '127946']
        self.ggrapher.parse_input()
        self.assertEqual(self.ggrapher.get_ancestors, True)
        self.assertEqual(self.ggrapher.get_descendants, False)
        self.assertEqual(self.ggrapher.verbose, False)
        self.assertEqual(self.ggrapher.write_filename, None)
        self.assertEqual(self.ggrapher.seed_ids, [127946])

        self.ggrapher.build_graph_complete(LocalDataGrabber)

        # Redirect stdout to capture output.
        stdout = sys.stdout
        stdout_intercept = StringIO.StringIO()
        sys.stdout = stdout_intercept
        self.ggrapher.generate_dot_file()
        sys.stdout = stdout

        expected = u"""digraph genealogy {
    graph [charset="utf-8"];
    node [shape=plaintext];
    edge [style=bold];

    127946 [label="Christian   Thomasius \\n(1672)"];
    137717 [label="Valentin  Alberti \\nUniversit\xe4t Leipzig (1678)"];
    137705 [label="Jakob  Thomasius \\nUniversit\xe4t Leipzig (1643)"];
    143630 [label="Friedrich  Leibniz"];

    137717 -> 127946;
    137705 -> 127946;
    143630 -> 137705;
}
"""
        self.assertEqual(stdout_intercept.getvalue().decode('utf-8'), expected)

    def test_end_to_end_descendants_stdout(self):
        """
        Complete test getting with descendants, writing the result to stdout.
        """
        sys.argv = ['geneagrapher', '-d', '79568']
        self.ggrapher.parse_input()
        self.assertEqual(self.ggrapher.get_ancestors, False)
        self.assertEqual(self.ggrapher.get_descendants, True)
        self.assertEqual(self.ggrapher.verbose, False)
        self.assertEqual(self.ggrapher.write_filename, None)
        self.assertEqual(self.ggrapher.seed_ids, [79568])

        self.ggrapher.build_graph_complete(LocalDataGrabber)

        # Redirect stdout to capture output.
        stdout = sys.stdout
        stdout_intercept = StringIO.StringIO()
        sys.stdout = stdout_intercept
        self.ggrapher.generate_dot_file()
        sys.stdout = stdout

        expected = u"""digraph genealogy {
    graph [charset="utf-8"];
    node [shape=plaintext];
    edge [style=bold];

    79568 [label="Ramdas  Kumaresan \\nUniversity of Rhode Island (1982)"];
    79562 [label="C. S. Ramalingam \\nUniversity of Rhode Island (1995)"];
    99457 [label="Yadong  Wang \\nUniversity of Rhode Island (2003)"];

    79568 -> 79562;
    79568 -> 99457;
}
"""
        self.assertEqual(stdout_intercept.getvalue().decode('utf-8'), expected)

    def test_end_to_end_self_file(self):
        """Complete test getting no ancestors or descendants and writing the
        # result to stdout."""
        outfname = 'outfile.test'
        sys.argv = ['geneagrapher', '-f', outfname, '30484']
        self.ggrapher.parse_input()
        self.assertEqual(self.ggrapher.get_ancestors, False)
        self.assertEqual(self.ggrapher.get_descendants, False)
        self.assertEqual(self.ggrapher.verbose, False)
        self.assertEqual(self.ggrapher.write_filename, outfname)
        self.assertEqual(self.ggrapher.seed_ids, [30484])

        self.ggrapher.build_graph_complete(LocalDataGrabber)
        self.ggrapher.generate_dot_file()

        expected = u"""digraph genealogy {
    graph [charset="utf-8"];
    node [shape=plaintext];
    edge [style=bold];

    30484 [label="Peter Chris Pappas \\nThe Pennsylvania State University \
(1982)"];

}
"""
        with open(outfname, 'r') as fin:
            self.assertEqual(fin.read().decode('utf-8'), expected)
        os.remove(outfname)

    def test_end_to_end_through_ggrapher_self_stdout(self):
        """Complete test calling ggrapher getting no ancestors or descendants
        and writing the result to stdout."""
        cache_fname = LocalDataGrabber.data_file(
            'end-to-end-30484')
        sys.argv = ['geneagrapher', '--cache-file', cache_fname, '30484']

        # Redirect stdout to capture output.
        stdout = sys.stdout
        stdout_intercept = StringIO.StringIO()
        sys.stdout = stdout_intercept
        geneagrapher.ggrapher()
        sys.stdout = stdout

        expected = u"""digraph genealogy {
    graph [charset="utf-8"];
    node [shape=plaintext];
    edge [style=bold];

    30484 [label="Peter Chris Pappas \\nThe Pennsylvania State University \
(1982)"];

}
"""
        self.assertEqual(stdout_intercept.getvalue().decode('utf-8'), expected)

if __name__ == '__main__':
    unittest.main()



from node import Node
from record import Record


class DuplicateNodeError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return self.value


class Graph(dict):
    """
    Class storing the representation of a genealogy graph.
    """

    def __init__(self, seed_nodes=None):
        """
        Graph class constructor.

        Parameters:
            seed_nodes: a set of Node objects representing the tree seed nodes
                (omit to create an empty graph)
        """
        dict.__init__(self)

        # Verify type of seed_nodes is what we expect.
        if seed_nodes is not None:
            if not isinstance(seed_nodes, set):
                raise TypeError(
                    "Unexpected argument type: expected set of \
Node objects for 'seed_nodes'"
                )
            for seed in seed_nodes:
                if not isinstance(seed, Node):
                    raise TypeError(
                        "Unexpected parameter type: expected set \
of Node objects for 'seed_nodes'"
                    )

        if seed_nodes is None:
            self.seeds = set()
        else:
            self.seeds = set([seed.get_id() for seed in seed_nodes])
            self.update([(seed.get_id(), seed) for seed in seed_nodes])

        self.supp_id = -1

    def has_node(self, id):
        """
        Check if the graph contains a node with the given id.
        """
        return id in self

    def get_node(self, id):
        """
        Return the node in the graph with given id. Returns
        None if no such node exists.
        """
        return self[id]

    def get_node_list(self):  # NOTE: this method is unused
        """
        Return a list of the nodes in the graph.
        """
        return self.keys()

    def add_node(self, name, institution, year, id, advisors, advisees, is_seed=False):
        """
        Add a new node to the graph if a matching node is not already
        present.
        """
        record = Record(name, institution, year, id)

        # Ancestors is the set of advisors already in the graph.
        graph_ancestors = set([advisor for advisor in advisors if advisor in self])
        # For each ancestor, add this node's id to the ancestor's descendant
        # set.
        for ancestor in graph_ancestors:
            self[ancestor].descendants.add(id)

        # Descendants is the set of advisees already in the graph.
        graph_descendants = set(
            [descendant for descendant in advisees if descendant in self]
        )
        # For each descendant, add this node's id to the descendant's
        # ancestor set.
        for descendant in graph_descendants:
            self[descendant].ancestors.add(id)

        node = Node(record, graph_ancestors, graph_descendants)
        self.add_node_object(node, is_seed)

    def add_node_object(self, node, is_seed=False):
        """
        Add a new node object to the graph if a node with the same id
        is not already present.
        """
        if node.get_id() is not None and self.has_node(node.get_id()):
            msg = "node with id {} already exists".format(node.get_id())
            raise DuplicateNodeError(msg)
        if node.get_id() is None:
            # Assign a "dummy" id.
            node.set_id(self.supp_id)
            self.supp_id -= 1
        self[node.get_id()] = node
        if self.seeds == set() or is_seed:
            self.seeds.add(node.get_id())

    def generate_dot_file(self, include_ancestors, include_descendants):
        """
        Return a string that contains the content of the Graphviz dotfile
        format for this graph.
        """
        if self.seeds == set():
            return ""

        queue = []
        for seed in self.seeds:
            queue.append(seed)
        edges = ""
        dotfile = ""

        dotfile += u"""digraph genealogy {
    graph [charset="utf-8"];
    node [shape=plaintext];
    edge [style=bold];\n\n"""

        printed_nodes = {}
        while len(queue) > 0:
            node_id = queue.pop(0)
            if not self.has_node(node_id):
                # Skip this id if a corresponding node is not present.
                continue
            if node_id in printed_nodes:
                # Skip this id because it is already printed.
                continue
            node = self.get_node(node_id)
            printed_nodes[node_id] = node

            sorted_ancestors = sorted(
                [a for a in node.ancestors if a in self],
                lambda x, y: cmp(
                    self[x].record.name.split()[-1], self[y].record.name.split()[-1]
                ),
            )
            sorted_descendants = sorted(
                [d for d in node.descendants if d in self],
                lambda x, y: cmp(
                    self[x].record.name.split()[-1], self[y].record.name.split()[-1]
                ),
            )

            if include_ancestors:
                # Add this node's advisors to queue.
                queue += sorted_ancestors

            if include_descendants:
                # Add this node's descendants to queue.
                queue += sorted_descendants

            # Print this node's information.
            nodestr = u'    {} [label="{}"];'.format(node_id, node)
            dotfile += nodestr

            # Store the connection information for this node.
            for advisor in sorted_ancestors:
                if self.has_node(advisor):
                    edgestr = "\n    {} -> {};".format(advisor, node_id)
                    edges += edgestr

            dotfile += "\n"

        # Now print the connections between the nodes.
        dotfile += edges

        dotfile += "\n}\n"
        return dotfile



import os
from time import time
import unittest
from local_data_grabber import LocalDataGrabber
from geneagrapher.cache_grabber import CacheGrabber
from geneagrapher.grabber import Grabber


class TestCacheGrabberMethods(unittest.TestCase):
    """Unit tests for the geneagrapher.CacheGrabber class."""
    def setUp(self):
        self.name = u"Carl Friedrich Gau\xdf"
        self.institution = u"Universit\xe4t Helmstedt"
        self.year = 1799
        self.advisors = set([18230])
        self.descendants = set([18603, 18233, 62547, 29642, 55175,
                                29458, 19953, 18232, 151876])

    def tearDown(self):
        try:
            os.remove('geneacache')
        except OSError:
            pass

    def test_init1(self):
        """Test constructor."""
        cache = CacheGrabber()
        self.assertEqual(cache.filename, 'geneacache')
        self.assertEqual(len(cache.cache), 0)
        self.assertIsInstance(cache.grabber, Grabber)
        self.assertEqual(cache.expiration_interval, 604800.)

    def test_init2(self):
        """Test constructor with non-default filename."""
        cache = CacheGrabber('mycachename')
        self.assertEqual(cache.filename, 'mycachename')
        self.assertEqual(len(cache.cache), 0)
        self.assertIsInstance(cache.grabber, Grabber)
        self.assertEqual(cache.expiration_interval, 604800.)
        os.remove('mycachename')

    def test_init3(self):
        """Test constructor with non-default record grabber."""
        cache = CacheGrabber(record_grabber=LocalDataGrabber)
        self.assertEqual(cache.filename, 'geneacache')
        self.assertEqual(len(cache.cache), 0)
        self.assertIsInstance(cache.grabber, LocalDataGrabber)
        self.assertEqual(cache.expiration_interval, 604800.)

    def test_init4(self):
        """Test constructor with non-default expiration interval."""
        cache = CacheGrabber(expiration_interval=1209600.)
        self.assertEqual(cache.filename, 'geneacache')
        self.assertEqual(len(cache.cache), 0)
        self.assertIsInstance(cache.grabber, Grabber)
        self.assertEqual(cache.expiration_interval, 1209600.)

    def test_close(self):
        """Test the close method."""
        cache = CacheGrabber(record_grabber=LocalDataGrabber)
        self.assertEqual(len(cache.cache), 0)
        cache.close()
        self.assertRaisesRegexp(ValueError,
                                'invalid operation on closed shelf',
                                len, cache.cache)

    def test_is_expired_false(self):
        """Test the is_expired method."""
        t = time()
        d = {'name': self.name, 'institution': self.institution,
             'year': self.year, 'advisors': self.advisors,
             'descendants': self.descendants, 'timestamp': t}
        with CacheGrabber() as cache:
            self.assertFalse(cache.is_expired(d))
            d['timestamp'] = time() - cache.expiration_interval + 20
            self.assertFalse(cache.is_expired(d))

    def test_is_expired_true(self):
        """Test the is_expired method."""
        with CacheGrabber() as cache:
            t = time() - cache.expiration_interval - 1
            d = {'name': self.name, 'institution': self.institution,
                 'year': self.year, 'advisors': self.advisors,
                 'descendants': self.descendants, 'timestamp': t}
            self.assertTrue(cache.is_expired(d))

    def test_context_manager(self):
        """Test the context manager methods."""
        with CacheGrabber(record_grabber=LocalDataGrabber) as cache:
            self.assertEqual(len(cache.cache), 0)
        self.assertRaisesRegexp(ValueError,
                                'invalid operation on closed shelf',
                                len, cache.cache)

    def test_get_record_bad(self):
        """Test the get_record method for a bad id."""
        with CacheGrabber(record_grabber=LocalDataGrabber) as cache:
            self.assertEqual(len(cache.cache), 0)
            self.assertRaisesRegexp(ValueError, 'Invalid id 999999999',
                                    cache.get_record, 999999999)
            self.assertEqual(len(cache.cache), 0)

    def test_get_record(self):
        """Test the get_record method for a good id."""
        with CacheGrabber(record_grabber=LocalDataGrabber) as cache:
            record = cache.get_record(18231)
            self.assertEqual(len(record), 6)
            self.assertEqual(record['name'], u"Carl Friedrich Gau\xdf")
            self.assertEqual(record['institution'],
                             u"Universit\xe4t Helmstedt")
            self.assertEqual(record['year'], 1799)
            self.assertEqual(record['advisors'], set([18230]))
            self.assertEqual(record['descendants'],  set([234500, 234374, 55175,
                                                          151876, 29642, 18603,
                                                          19953, 217618, 62547,
                                                          225908, 166471, 18232,
                                                          18233, 234267,
                                                          165758]))
            self.assertEqual(record['message'], u"cache miss")
            self.assertEqual(len(cache.cache), 1)

            # Make the request again and verify the cached version is returned.
            d = cache.cache['18231']
            d['institution'] = u'Rigged for test'
            cache.cache['18231'] = d
            record = cache.get_record(18231)
            self.assertEqual(len(record), 6)
            self.assertEqual(record['name'], u"Carl Friedrich Gau\xdf")
            self.assertEqual(record['institution'], u"Rigged for test")
            self.assertEqual(record['year'], 1799)
            self.assertEqual(record['advisors'], set([18230]))
            self.assertEqual(record['descendants'], set([234500, 234374, 55175,
                                                         151876, 29642, 18603,
                                                         19953, 217618, 62547,
                                                         225908, 166471, 18232,
                                                         18233, 234267,
                                                         165758]))
            self.assertEqual(record['message'], u"cache hit")
            self.assertEqual(len(cache.cache), 1)

        with CacheGrabber(record_grabber=LocalDataGrabber) as cache:
            # Redo the last request, this time with a newly-loaded instance
            # of the cache from disk.
            self.assertEqual(len(cache.cache), 1)
            d = cache.cache['18231']
            d['institution'] = u'Rigged for test'
            cache.cache['18231'] = d
            record = cache.get_record(18231)
            self.assertEqual(len(record), 6)
            self.assertEqual(record['name'], u"Carl Friedrich Gau\xdf")
            self.assertEqual(record['institution'], u"Rigged for test")
            self.assertEqual(record['year'], 1799)
            self.assertEqual(record['advisors'], set([18230]))
            self.assertEqual(record['descendants'], set([234500, 234374, 55175,
                                                         151876, 29642, 18603,
                                                         19953, 217618, 62547,
                                                         225908, 166471, 18232,
                                                         18233, 234267,
                                                         165758]))
            self.assertEqual(record['message'], u"cache hit")
            self.assertEqual(len(cache.cache), 1)

            # Make another request, this time with the cached entry expired,
            # and verify a new version is retrieved.
            d['timestamp'] = time() - cache.expiration_interval - 1
            cache.cache['18231'] = d
            record = cache.get_record(18231)
            self.assertEqual(len(record), 6)
            self.assertEqual(record['name'], u"Carl Friedrich Gau\xdf")
            self.assertEqual(record['institution'],
                             u"Universit\xe4t Helmstedt")
            self.assertEqual(record['year'], 1799)
            self.assertEqual(record['advisors'], set([18230]))
            self.assertEqual(record['descendants'], set([234500, 234374, 55175,
                                                         151876, 29642, 18603,
                                                         19953, 217618, 62547,
                                                         225908, 166471, 18232,
                                                         18233, 234267,
                                                         165758]))
            self.assertEqual(record['message'], u"cache miss")
            self.assertEqual(len(cache.cache), 1)

    def test_is_in_cache(self):
        """Test the is_in_cache method."""
        d = {'name': self.name, 'institution': self.institution,
             'year': self.year, 'advisors': self.advisors,
             'descendants': self.descendants, 'timestamp': time()}
        with CacheGrabber(record_grabber=LocalDataGrabber) as cache:
            self.assertFalse(cache.is_cached(self.id))
            cache.cache[str(self.id)] = d
            self.assertTrue(cache.is_cached(self.id))
            new_timestamp = time() - cache.expiration_interval - 1
            d['timestamp'] = new_timestamp
            cache.cache[str(self.id)] = d
            self.assertFalse(cache.is_cached(self.id))

    def test_load_into_cache(self):
        """Test the load_into_cache method."""
        with CacheGrabber(record_grabber=LocalDataGrabber) as cache:
            self.assertEqual(len(cache.cache), 0)
            new_record = {'name': self.name, 'institution': self.institution,
                          'year': self.year, 'advisors': self.advisors,
                          'descendants': self.descendants}
            cache.load_into_cache(self.id, new_record)
            self.assertEqual(len(cache.cache), 1)
            record = cache.cache[str(self.id)]
            self.assertEqual(record['name'], self.name)
            self.assertEqual(record['institution'], self.institution)
            self.assertEqual(record['year'], self.year)
            self.assertEqual(record['advisors'], self.advisors)
            self.assertEqual(record['descendants'], self.descendants)
            self.assertTrue(time() - record['timestamp'] < 20)

            # Insert the same record a second time to verify replacement
            # behavior.
            self.assertEqual(len(cache.cache), 1)
            record = cache.cache[str(self.id)]
            self.assertEqual(record['name'], self.name)
            self.assertEqual(record['institution'], self.institution)
            self.assertEqual(record['year'], self.year)
            self.assertEqual(record['advisors'], self.advisors)
            self.assertEqual(record['descendants'], self.descendants)
            self.assertTrue(time() - record['timestamp'] < 20)


if __name__ == '__main__':
    unittest.main()



import shelve
from time import time
from grabber import Grabber


class CacheGrabber:
    def __init__(
        self, filename="geneacache", record_grabber=Grabber, expiration_interval=604800.
    ):
        self.filename = filename
        self.grabber = record_grabber()
        self.expiration_interval = float(expiration_interval)
        self.cache = shelve.open(filename)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

    def close(self):
        """Close the cache. All methods after calling this will raise
        ValueError."""
        self.cache.close()

    def is_expired(self, record):
        """Returns True if the given record is expired."""
        return time() - record["timestamp"] > self.expiration_interval

    def get_record(self, id):
        """Return information for the mathematician associated with the given
        id."""
        id_str = str(id)
        if self.is_cached(id_str):
            record = self.cache[id_str]
            record["message"] = "cache hit"
        else:
            record = self.grabber.get_record(id)
            self.load_into_cache(id, record)
            record["message"] = "cache miss"
        del (record["timestamp"])
        return record

    def is_cached(self, id):
        """Return True if an item with the given id is in the cache and has
        not expired."""
        return str(id) in self.cache and not self.is_expired(self.cache[str(id)])

    def load_into_cache(self, id, record):
        """Insert a new record into the cache.

        If the record already exists, its values are replaced with the values
        provided as input to this method."""
        record["timestamp"] = time()
        self.cache[str(id)] = record



from argparse import ArgumentParser
from collections import deque
import pkg_resources
import sys
from cache_grabber import CacheGrabber
from graph import Graph
from grabber import Grabber


class Geneagrapher:
    """
    A class for building Graphviz "dot" files for math genealogies
    extracted from the Mathematics Genealogy Project website.
    """

    def __init__(self):
        self.graph = Graph()
        self.seed_ids = []
        self.get_ancestors = False
        self.get_descendants = False
        self.verbose = False
        self.write_filename = None
        self.use_cache = True
        self.cache_file = "geneacache"

    def parse_input(self):
        """
        Parse command-line information.
        """
        pkg_env = pkg_resources.Environment()
        description = 'Create a Graphviz "dot" file for a mathematics \
genealogy, where ID is a record identifier from the Mathematics Genealogy \
Project. Multiple IDs may be passed.'
        self.parser = ArgumentParser(description=description)

        try:
            version = "{}".format(pkg_env[self.__module__.split(".")[0]][0].version)
        except IndexError:
            version = "dev"
        self.parser.add_argument(
            "--version", action="version", version="%(prog)s {0}".format(version)
        )
        self.parser.add_argument(
            "-f",
            "--file",
            dest="filename",
            help="write output to FILE [default: stdout]",
            metavar="FILE",
            default=None,
        )
        self.parser.add_argument(
            "-a",
            "--with-ancestors",
            action="store_true",
            dest="get_ancestors",
            default=False,
            help="retrieve ancestors of IDs and include \
in graph",
        )
        self.parser.add_argument(
            "-d",
            "--with-descendants",
            action="store_true",
            dest="get_descendants",
            default=False,
            help="retrieve descendants of IDs and \
include in graph",
        )
        self.parser.add_argument(
            "--disable-cache",
            action="store_false",
            dest="use_cache",
            default=True,
            help="do not store records in local cache",
        )
        self.parser.add_argument(
            "--cache-file",
            dest="cache_file",
            help="write cache to FILE [default: \
geneacache]",
            metavar="FILE",
            default="geneacache",
        )
        self.parser.add_argument(
            "-v",
            "--verbose",
            action="store_true",
            dest="verbose",
            default=False,
            help="list nodes being retrieved",
        )
        self.parser.add_argument(
            "ids", metavar="ID", type=int, nargs="+", help="mathematician record ID"
        )

        args = self.parser.parse_args()

        self.get_ancestors = args.get_ancestors
        self.get_descendants = args.get_descendants
        self.verbose = args.verbose
        self.write_filename = args.filename
        self.use_cache = args.use_cache
        self.cache_file = args.cache_file
        self.seed_ids = [int(arg) for arg in args.ids]

    def build_graph_portion(self, grab_queue, is_seed, grabber, **kwargs):
        """Handle grabbing and storing nodes in the graph. Depending on the
        arguments, this method handles seed nodes, ancestors, or
        descendants."""
        while len(grab_queue) != 0:
            id = grab_queue.popleft()
            if not self.graph.has_node(id):
                # Then this information has not yet been grabbed.
                if self.verbose:
                    sys.stdout.write("Grabbing record #{}...".format(id))
                record = grabber.get_record(id)
                if self.verbose:
                    if "message" in record:
                        print record["message"]
                    else:
                        print
                self.graph.add_node(
                    record["name"],
                    record["institution"],
                    record["year"],
                    id,
                    record["advisors"],
                    record["descendants"],
                    is_seed,
                )
                if self.get_ancestors and "ancestor_queue" in kwargs:
                    kwargs["ancestor_queue"].extend(record["advisors"])
                if self.get_descendants and "descendant_queue" in kwargs:
                    kwargs["descendant_queue"].extend(record["descendants"])

    def build_graph_complete(self, record_grabber=Grabber, **kwargs):
        """
        Populate the graph member by grabbing the mathematician
        pages and extracting relevant data.
        """
        seed_queue = deque(self.seed_ids)
        ancestor_queue = deque()
        descendant_queue = deque()
        with record_grabber(**kwargs) as grabber:
            # Grab "seed" nodes.
            self.build_graph_portion(
                seed_queue,
                True,
                grabber,
                ancestor_queue=ancestor_queue,
                descendant_queue=descendant_queue,
            )

            # Grab ancestors of seed nodes.
            if self.get_ancestors:
                self.build_graph_portion(
                    ancestor_queue, False, grabber, ancestor_queue=ancestor_queue
                )

            # Grab descendants of seed nodes.
            if self.get_descendants:
                self.build_graph_portion(
                    descendant_queue, False, grabber, descendant_queue=descendant_queue
                )

    def build_graph(self):
        """Call the graph builder method with the correct arguments, based
        on the command-line arguments."""
        if self.use_cache:
            record_grabber = CacheGrabber
        else:
            record_grabber = Grabber
        self.build_graph_complete(record_grabber, filename=self.cache_file)

    def generate_dot_file(self):
        dotfile = self.graph.generate_dot_file(self.get_ancestors, self.get_descendants)
        dotfile = dotfile.encode("utf-8", "replace")
        if self.write_filename is not None:
            outfile = open(self.write_filename, "w")
            outfile.write(dotfile)
            outfile.close()
        else:
            print dotfile,


def ggrapher():
    """Function to run the Geneagrapher. This is the function called when
    the ggrapher script is run."""
    ggrapher = Geneagrapher()
    ggrapher.parse_input()

    try:
        ggrapher.build_graph()
    except ValueError as e:
        print e
    ggrapher.generate_dot_file()


if __name__ == "__main__":
    ggrapher()



import os
import sys
from bs4 import BeautifulSoup
from geneagrapher.grabber import get_record_from_tree


class LocalDataGrabber:
    """A class for grabbing locally-cached test data."""
    def __init__(self):
        pass

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        pass

    @classmethod
    def data_file(cls, filename):
        """Return the absolute path to the data file with given name."""
        return os.path.join(cls.data_path, filename)

    def get_record(self, id):
        """Load the local data for the given id and use Grabber's functionas
        to extract the record data."""
        with open(self.data_file('{0}.html'.format(id)), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
        return get_record_from_tree(soup, id)


file_path = os.path.abspath(__file__)
LocalDataGrabber.data_path = os.path.join(os.path.dirname(file_path),
                                          'testdata')



import unittest
from geneagrapher.graph import DuplicateNodeError, Graph, Node, Record


class TestGraphMethods(unittest.TestCase):
    """Unit tests for the Graph class."""
    def setUp(self):
        self.record1 = Record(u"Carl Friedrich Gau\xdf",
                              u"Universit\xe4t Helmstedt", 1799, 18231)
        self.node1 = Node(self.record1, set(), set())
        self.graph1 = Graph(set([self.node1]))

    def test_init_empty(self):
        """Test the constructor with an empty seeds set."""
        graph = Graph()
        self.assertEqual(graph.seeds, set())

    def test_init(self):
        """Test the constructor."""
        self.assertEqual(self.graph1.seeds, set([18231]))
        self.assertEqual(self.graph1.keys(), [18231])
        self.assertEqual(self.graph1[18231], self.node1)

    def test_init_bad_seeds(self):
        """
        Test the constructor when passed a bad type for the seeds parameter.
        """
        self.assertRaises(TypeError, Graph, 3)

    def test_has_node_true(self):
        """Test the has_node() method for a True case."""
        self.assertEqual(self.graph1.has_node(18231), True)

    def test_has_node_false(self):
        """Test the has_node() method for a False case."""
        self.assertEqual(self.graph1.has_node(1), False)

    def test_contains_node_true(self):
        """Test the __contains__() method for a True case."""
        self.assertEqual(18231 in self.graph1, True)

    def test_contains_node_false(self):
        """Test the __contains__() method for a False case."""
        self.assertEqual(1 in self.graph1, False)

    def test_get_node(self):
        """Test the get_node() method."""
        node = self.graph1.get_node(18231)
        self.assert_(node == self.node1)

    def test_get_node_not_found(self):
        """
        Test the get_node() method for a case where the node does not exist.
        """
        self.assertRaises(KeyError, self.graph1.get_node, 1)

    def test_get_node_list(self):
        """Test the get_node_list() method."""
        self.assertEqual(self.graph1.get_node_list(), [18231])

    def test_get_node_list_empty(self):
        """Test the get_node_list() method for an empty graph."""
        graph = Graph()
        self.assertEqual(graph.get_node_list(), [])

    def test_add_node(self):
        """Test the add_node() method."""
        self.graph1.add_node("Leonhard Euler", "Universitaet Basel", 1726,
                             38586, set(), set())
        self.assertEqual([38586, 18231], self.graph1.get_node_list())
        self.assertEqual(self.graph1.seeds, set([18231]))

    def test_add_second_node_seed(self):
        """Test the add_node() method when adding a second node and
        marking it as a seed node."""
        self.graph1.add_node("Leonhard Euler", "Universitaet Basel", 1726,
                             38586, set(), set(), True)
        self.assertEqual([38586, 18231], self.graph1.get_node_list())
        self.assertEqual(self.graph1.seeds, set([18231, 38586]))

    def test_add_node_seed(self):
        """Test the add_node() method when no seeds exist."""
        graph = Graph()
        self.assertEqual(graph.seeds, set())
        graph.add_node("Leonhard Euler", "Universitaet Basel", 1726,
                       38586, set(), set())
        self.assertEqual(graph.seeds, set([38586]))

    def test_add_node_already_present(self):
        """Test for expected exception when adding a duplicate node."""
        self.graph1.add_node("Leonhard Euler", "Universitaet Basel", 1726,
                             38586, set(), set())
        self.assertEqual([38586, 18231], self.graph1.get_node_list())
        self.assertRaises(DuplicateNodeError, self.graph1.add_node,
                          "Leonhard Euler", "Universitaet Basel",
                          1726, 38586, set(), set())

        try:
            self.graph1.add_node("Leonhard Euler", "Universitaet Basel",
                                 1726, 38586, set(), set())
        except DuplicateNodeError as e:
            self.assertEqual(str(e),
                             "node with id {} already exists".format(38586))
        else:
            self.fail()

    def test_add_node_object(self):
        """Test the add_node_object() method."""
        record = Record("Leonhard Euler", "Universitaet Basel", 1726, 38586)
        node = Node(record, set(), set())
        self.graph1.add_node_object(node)
        self.assertEqual([38586, 18231], self.graph1.get_node_list())
        self.assertEqual(self.graph1.seeds, set([18231]))

    def test_generate_dot_file(self):
        """Test the generate_dot_file() method."""
        dotfileexpt = u"""digraph genealogy {
    graph [charset="utf-8"];
    node [shape=plaintext];
    edge [style=bold];

    18231 [label="Carl Friedrich Gau\xdf \\nUniversit\xe4t Helmstedt (1799)"];

}
"""
        dotfile = self.graph1.generate_dot_file(True, False)
        self.assertEqual(dotfile, dotfileexpt)

    def test_generate_dot_file2(self):
        """Test the generate_dot_file() method.

        This is a larger example than test_generate_dot_file()."""
        graph = Graph()
        graph.add_node(u"Carl Friedrich Gau\xdf", u"Universit\xe4t Helmstedt",
                       1799, 18231, set([18230]), set())
        graph.add_node(u"Johann Friedrich Pfaff",
                       u"Georg-August-Universit\xe4t Goettingen", 1786, 18230,
                       set([66476]), set([18231]))
        graph.add_node(u"Abraham Gotthelf Kaestner", u"Universit\xe4t Leipzig",
                       1739, 66476, set([57670]), set([18230]))
        graph.add_node(u"Christian August Hausen",
                       u"Martin-Luther-Universit\xe4t Halle-Wittenberg", 1713,
                       57670, set([72669]), set([66476]))
        graph.add_node(u"Johann Christoph Wichmannshausen",
                       u"Universit\xe4t Leipzig", 1685, 72669, set([21235]),
                       set([57670]))
        graph.add_node(u"Otto Mencke", u"Universit\xe4t Leipzig", 1665, 21235,
                       set(), set([72669]))

        dotfileexpt = u"""digraph genealogy {
    graph [charset="utf-8"];
    node [shape=plaintext];
    edge [style=bold];

    18231 [label="Carl Friedrich Gau\xdf \\nUniversit\xe4t Helmstedt (1799)"];
    18230 [label="Johann Friedrich Pfaff \\nGeorg-August-Universit\xe4t \
Goettingen (1786)"];
    66476 [label="Abraham Gotthelf Kaestner \\nUniversit\xe4t Leipzig (1739)"];
    57670 [label="Christian August Hausen \\nMartin-Luther-Universit\xe4t \
Halle-Wittenberg (1713)"];
    72669 [label="Johann Christoph Wichmannshausen \\nUniversit\xe4t Leipzig \
(1685)"];
    21235 [label="Otto Mencke \\nUniversit\xe4t Leipzig (1665)"];

    18230 -> 18231;
    66476 -> 18230;
    57670 -> 66476;
    72669 -> 57670;
    21235 -> 72669;
}
"""
        dotfile = graph.generate_dot_file(True, False)
        self.assertEqual(dotfile, dotfileexpt)

    def test_incremental_ancestor_descendant_check(self):
        """Test the contents of the ancestors and descendants members of a
        graph's nodes as they are added."""
        graph = Graph()
        graph.add_node(u"Carl Friedrich Gau\xdf", u"Universit\xe4t Helmstedt",
                       1799, 18231, set([18230]), set([18603, 18233, 62547]))
        node1 = graph[18231]
        self.assertEqual(node1.ancestors, set())
        self.assertEqual(node1.descendants, set())

        graph.add_node(u"Johann Friedrich Pfaff",
                       u"Georg-August-Universit\xe4t Goettingen", 1786, 18230,
                       set([66476]), set([18231]))
        node2 = graph[18230]
        self.assertEqual(node1.ancestors, set([18230]))
        self.assertEqual(node1.descendants, set())
        self.assertEqual(node2.ancestors, set())
        self.assertEqual(node2.descendants, set([18231]))

        graph.add_node(u"Abraham Gotthelf Kaestner", u"Universit\xe4t Leipzig",
                       1739, 66476, set([57670]), set([18230]))
        node3 = graph[66476]
        self.assertEqual(node1.ancestors, set([18230]))
        self.assertEqual(node1.descendants, set())
        self.assertEqual(node2.ancestors, set([66476]))
        self.assertEqual(node2.descendants, set([18231]))
        self.assertEqual(node3.ancestors, set())
        self.assertEqual(node3.descendants, set([18230]))

        graph.add_node(u"Christian August Hausen",
                       u"Martin-Luther-Universit\xe4t Halle-Wittenberg", 1713,
                       57670, set([72669]), set([66476]))
        node4 = graph[57670]
        self.assertEqual(node1.ancestors, set([18230]))
        self.assertEqual(node1.descendants, set())
        self.assertEqual(node2.ancestors, set([66476]))
        self.assertEqual(node2.descendants, set([18231]))
        self.assertEqual(node3.ancestors, set([57670]))
        self.assertEqual(node3.descendants, set([18230]))
        self.assertEqual(node4.ancestors, set())
        self.assertEqual(node4.descendants, set([66476]))

        graph.add_node(u"Johann Christoph Wichmannshausen",
                       u"Universit\xe4t Leipzig", 1685, 72669, set([21235]),
                       set([57670]))
        node5 = graph[72669]
        self.assertEqual(node1.ancestors, set([18230]))
        self.assertEqual(node1.descendants, set())
        self.assertEqual(node2.ancestors, set([66476]))
        self.assertEqual(node2.descendants, set([18231]))
        self.assertEqual(node3.ancestors, set([57670]))
        self.assertEqual(node3.descendants, set([18230]))
        self.assertEqual(node4.ancestors, set([72669]))
        self.assertEqual(node4.descendants, set([66476]))
        self.assertEqual(node5.ancestors, set())
        self.assertEqual(node5.descendants, set([57670]))

        graph.add_node(u"Otto Mencke", u"Universit\xe4t Leipzig", 1665, 21235,
                       set(), set([72669]))
        node6 = graph[21235]
        self.assertEqual(node1.ancestors, set([18230]))
        self.assertEqual(node1.descendants, set())
        self.assertEqual(node2.ancestors, set([66476]))
        self.assertEqual(node2.descendants, set([18231]))
        self.assertEqual(node3.ancestors, set([57670]))
        self.assertEqual(node3.descendants, set([18230]))
        self.assertEqual(node4.ancestors, set([72669]))
        self.assertEqual(node4.descendants, set([66476]))
        self.assertEqual(node5.ancestors, set([21235]))
        self.assertEqual(node5.descendants, set([57670]))
        self.assertEqual(node6.ancestors, set())
        self.assertEqual(node6.descendants, set([72669]))

    def test_incremental_ancestor_descendant_check2(self):
        """Test the contents of the ancestors and descendants members of a
        # graph's nodes as they are added inserted in a different ofder than
        # in the previous test."""
        graph = Graph()
        graph.add_node(u"Abraham Gotthelf Kaestner", u"Universit\xe4t Leipzig",
                       1739, 66476, set([57670]), set([18230]))
        node1 = graph[66476]
        self.assertEqual(node1.ancestors, set())
        self.assertEqual(node1.descendants, set())

        graph.add_node(u"Johann Friedrich Pfaff",
                       u"Georg-August-Universit\xe4t Goettingen", 1786, 18230,
                       set([66476]), set([18231]))
        node2 = graph[18230]
        self.assertEqual(node1.ancestors, set())
        self.assertEqual(node1.descendants, set([18230]))
        self.assertEqual(node2.ancestors, set([66476]))
        self.assertEqual(node2.descendants, set())

        graph.add_node(u"Christian August Hausen",
                       u"Martin-Luther-Universit\xe4t Halle-Wittenberg", 1713,
                       57670, set([72669]), set([66476]))
        node3 = graph[57670]
        self.assertEqual(node1.ancestors, set([57670]))
        self.assertEqual(node1.descendants, set([18230]))
        self.assertEqual(node2.ancestors, set([66476]))
        self.assertEqual(node2.descendants, set())
        self.assertEqual(node3.ancestors, set())
        self.assertEqual(node3.descendants, set([66476]))

        graph.add_node(u"Johann Christoph Wichmannshausen",
                       u"Universit\xe4t Leipzig", 1685, 72669, set([21235]),
                       set([57670]))
        node4 = graph[72669]
        self.assertEqual(node1.ancestors, set([57670]))
        self.assertEqual(node1.descendants, set([18230]))
        self.assertEqual(node2.ancestors, set([66476]))
        self.assertEqual(node2.descendants, set())
        self.assertEqual(node3.ancestors, set([72669]))
        self.assertEqual(node3.descendants, set([66476]))
        self.assertEqual(node4.ancestors, set())
        self.assertEqual(node4.descendants, set([57670]))

        graph.add_node(u"Otto Mencke", u"Universit\xe4t Leipzig", 1665, 21235,
                       set(), set([72669]))
        node5 = graph[21235]
        self.assertEqual(node1.ancestors, set([57670]))
        self.assertEqual(node1.descendants, set([18230]))
        self.assertEqual(node2.ancestors, set([66476]))
        self.assertEqual(node2.descendants, set())
        self.assertEqual(node3.ancestors, set([72669]))
        self.assertEqual(node3.descendants, set([66476]))
        self.assertEqual(node4.ancestors, set([21235]))
        self.assertEqual(node4.descendants, set([57670]))
        self.assertEqual(node5.ancestors, set())
        self.assertEqual(node5.descendants, set([72669]))

        graph.add_node(u"Carl Friedrich Gau\xdf", u"Universit\xe4t Helmstedt",
                       1799, 18231, set([18230]), set([18603, 18233, 62547]))
        node6 = graph[18231]
        self.assertEqual(node1.ancestors, set([57670]))
        self.assertEqual(node1.descendants, set([18230]))
        self.assertEqual(node2.ancestors, set([66476]))
        self.assertEqual(node2.descendants, set([18231]))
        self.assertEqual(node3.ancestors, set([72669]))
        self.assertEqual(node3.descendants, set([66476]))
        self.assertEqual(node4.ancestors, set([21235]))
        self.assertEqual(node4.descendants, set([57670]))
        self.assertEqual(node5.ancestors, set())
        self.assertEqual(node5.descendants, set([72669]))
        self.assertEqual(node6.ancestors, set([18230]))
        self.assertEqual(node6.descendants, set())

if __name__ == '__main__':
    unittest.main()



import unittest
from geneagrapher.graph import Record


# Unit tests for graph-related classes.
class TestRecordMethods(unittest.TestCase):
    """Unit tests for the Record class."""
    def setUp(self):
        self.record = Record(u"Carl Friedrich Gau\xdf",
                             u"Universit\xe4t Helmstedt", 1799, 18231)

    def test_init(self):
        """Test the constructor."""
        record = Record("Carl Friedrich Gauss", "Universitaet Helmstedt", 1799,
                        18231)
        self.assertEqual(record.name, "Carl Friedrich Gauss")
        self.assertEqual(record.institution, "Universitaet Helmstedt")
        self.assertEqual(record.year, 1799)
        self.assertEqual(record.id, 18231)

    def test_init_bad_name(self):
        """Test constructor with bad 'name' parameter."""
        self.assertRaises(TypeError, Record, 1, "Universitaet Helmstedt", 1799,
                          18231)

    def test_init_bad_institution(self):
        """Test constructor with bad 'institution' parameter."""
        self.assertRaises(TypeError, Record, "Carl Friedrich Gauss", 1, 1799,
                          18231)

    def test_init_bad_year(self):
        """Test constructor with bad 'year' parameter."""
        self.assertRaises(TypeError, Record, "Carl Friedrich Gauss",
                          "Universitaet Helmstedt", "1799", 18231)

    def test_init_bad_id(self):
        """Test constructor with bad 'id' parameter."""
        self.assertRaises(TypeError, Record, "Carl Friedrich Gauss",
                          "Universitaet Helmstedt", 1799, "18231")

    def test_cmp_equal(self):
        """Verify two 'equal' records are compared correctly."""
        record1 = Record("Carl Friedrich Gauss", "Universitaet Helmstedt",
                         1799, 18231)
        record2 = Record("Carl Friedrich Gauss", "Universitaet Helmstedt",
                         1799, 18231)
        self.assert_(record1 == record2)

    def test_cmp_unequal(self):
        """Verify two 'unequal' records are compared correctly."""
        record1 = Record("Carl Friedrich Gauss", "Universitaet Helmstedt",
                         1799, 18231)
        record2 = Record("Leonhard Euler", "Universitaet Basel", 1726, 38586)
        self.assert_(record1 < record2)

    def test_has_institution_yes(self):
        """Verify has_institution() method returns True when the conditions are
        right."""
        record = Record("Carl Friedrich Gauss", "Universitaet Helmstedt", 1799,
                        18231)
        self.assert_(record.has_institution())

    def test_has_institution_no(self):
        """Verify has_institution() method returns False when the conditions
        are right."""
        record = Record("Carl Friedrich Gauss", None, 1799, 18231)
        self.assert_(not record.has_institution())

    def test_has_year_yes(self):
        """
        Verify has_year() method returns True when the conditions are right.
        """
        record = Record("Carl Friedrich Gauss", "Universitaet Helmstedt", 1799,
                        18231)
        self.assert_(record.has_year())

    def test_has_year_no(self):
        """
        Verify has_year() method returns False when the conditions are right.
        """
        record = Record("Carl Friedrich Gauss", "Universitaet Helmstedt", None,
                        18231)
        self.assert_(not record.has_year())

    def test_unicode_full(self):
        """Test __unicode__() method for complete record."""
        recstr = self.record.__unicode__()
        recstrexpt = u"Carl Friedrich Gau\xdf \\nUniversit\xe4t Helmstedt \
(1799)"
        self.assertEqual(recstr, recstrexpt)

    def test_unicode_no_year(self):
        """Test __unicode__() method for record without year."""
        record = Record(self.record.name, self.record.institution, None, 18231)
        recstr = record.__unicode__()
        recstrexpt = u"Carl Friedrich Gau\xdf \\nUniversit\xe4t Helmstedt"
        self.assertEqual(recstr, recstrexpt)

    def test_unicode_no_inst(self):
        """Test __unicode__() method for record without institution."""
        record = Record(self.record.name, None, 1799, 18231)
        recstr = record.__unicode__()
        recstrexpt = u"Carl Friedrich Gau\xdf \\n(1799)"
        self.assertEqual(recstr, recstrexpt)

    def test_unicode_no_inst_no_id(self):
        """Test __unicode__() method for record without institution or year."""
        record = Record(self.record.name, None, None, 18231)
        recstr = record.__unicode__()
        recstrexpt = u"Carl Friedrich Gau\xdf"
        self.assertEqual(recstr, recstrexpt)

if __name__ == '__main__':
    unittest.main()



import os
import sys
from bs4 import BeautifulSoup
import unittest
from geneagrapher.grabber import *
from local_data_grabber import LocalDataGrabber


class TestGrabberMethods(unittest.TestCase):
    def data_file(self, filename):
        """Return the absolute path to the data file with given name."""
        return LocalDataGrabber.data_file(filename)

    def test_init(self):
        """Test constructor."""
        grabber = Grabber()
        self.assertIsInstance(grabber, Grabber)

    def test_get_record_bad(self):
        """Verify exception thrown from get_record() method for bad id."""
        grabber = Grabber()
        self.assertRaises(ValueError, grabber.get_record, 999999999)

        try:
            grabber.get_record(999999999)
        except ValueError as e:
            self.assertEqual(str(e), "Invalid id 999999999")
        else:
            self.fail()

    def test_get_record_all_fields(self):
        """Test the get_record() method for a record containing all fields."""
        grabber = Grabber()
        record = grabber.get_record(18231)
        self.assertEqual(len(record), 5)
        self.assertEqual(record['name'], u"Carl Friedrich Gau\xdf")
        self.assertEqual(record['institution'], u"Universit\xe4t Helmstedt")
        self.assertEqual(record['year'], 1799)
        self.assertEqual(record['advisors'], set([18230]))
        self.assertEqual(record['descendants'], set([234500, 234374, 55175,
                                                     151876, 29642, 18603,
                                                     19953, 217618, 62547,
                                                     225908, 166471, 18232,
                                                     18233, 234267, 165758]))

    def test_get_record_from_tree_bad(self):
        """Verify exception thrown from get_record_from_tree() method for bad
        id."""
        with open(self.data_file('999999999.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
        self.assertRaises(ValueError, get_record_from_tree, soup, 999999999)

        try:
            get_record_from_tree(soup, 999999999)
        except ValueError as e:
            self.assertEqual(str(e), "Invalid id 999999999")
        else:
            self.fail()

    def test_get_record_from_tree_all_fields(self):
        """Test the get_record_from_tree() method for a record containing all
        fields."""
        with open(self.data_file('18231.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
        record = get_record_from_tree(soup, 18231)
        self.assertEqual(len(record), 5)
        self.assertEqual(record['name'], u"Carl Friedrich Gau\xdf")
        self.assertEqual(record['institution'], u"Universit\xe4t Helmstedt")
        self.assertEqual(record['year'], 1799)
        self.assertEqual(record['advisors'], set([18230]))
        self.assertEqual(record['descendants'], set([234500, 234374, 55175,
                                                     151876, 29642, 18603,
                                                     19953, 217618, 62547,
                                                     225908, 166471, 18232,
                                                     18233, 234267, 165758]))

        # Verify calling get_record_from_tree() twice does not have side
        # effect.
        record = get_record_from_tree(soup, 18231)
        self.assertEqual(len(record), 5)
        self.assertEqual(record['name'], u"Carl Friedrich Gau\xdf")
        self.assertEqual(record['institution'], u"Universit\xe4t Helmstedt")
        self.assertEqual(record['year'], 1799)
        self.assertEqual(record['advisors'], set([18230]))
        self.assertEqual(record['descendants'], set([234500, 234374, 55175,
                                                     151876, 29642, 18603,
                                                     19953, 217618, 62547,
                                                     225908, 166471, 18232,
                                                     18233, 234267, 165758]))

    def test_has_record_true(self):
        """Test the has_record() method with a tree containing a
        mathematician record."""
        with open(self.data_file('137717.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertTrue(has_record(soup))

    def test_has_record_false(self):
        """Test the record_exists() method with a tree not containing a
        mathematician record."""
        with open(self.data_file('137717.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertTrue(has_record(soup))

    def test_get_name(self):
        """Test the get_name() method."""
        with open(self.data_file('137717.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertEqual(u"Valentin  Alberti", get_name(soup))

    def test_get_name_slash_l(self):
        """Test the get_name() method for a record containing a non-ASCII
        character."""
        with open(self.data_file('7383.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertEqual(u"W\u0142adys\u0142aw Hugo Dyonizy Steinhaus",
                             get_name(soup))

    def test_get_institution(self):
        """Test the get_institution() method for a record with an
        institution."""
        with open(self.data_file('137717.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertEqual(u"Universit\xe4t Leipzig", get_institution(soup))

    def test_get_institution_no_institution(self):
        """Test the get_institution() method for a record with no
        institution."""
        with open(self.data_file('52965.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertIsNone(get_institution(soup))

    def test_get_year(self):
        """Test the get_year() method for a record with a graduation year."""
        with open(self.data_file('137717.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertEqual(get_year(soup), 1678)

    def test_get_year_no_year(self):
        """Test the get_year() method for a record with no graduation year."""
        with open(self.data_file('53658.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertIsNone(get_year(soup))

    def test_get_advisors(self):
        """Test the get_advisors() method for a record with advisors."""
        with open(self.data_file('18231.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertEqual(get_advisors(soup), set([18230]))

    def test_get_advisors_multiple(self):
        """Test the get_advisors() method for a record with multiple
        advisors."""
        with open(self.data_file('19964.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertEqual(get_advisors(soup), set([17946, 47064]))

    def test_get_advisors_no_advisors(self):
        """Test the get_advisors() method for a record with no advisors."""
        with open(self.data_file('137717.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertEqual(get_advisors(soup), set([]))

    def test_get_descendants(self):
        """Test the get_descendants() method for a record with descendants."""
        expected_descendants = set([234500, 234374, 55175, 151876, 29642, 18603,
                                    19953, 217618, 62547, 225908, 166471, 18232,
                                    18233, 234267, 165758])
        with open(self.data_file('18231.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertEqual(get_descendants(soup), expected_descendants)

    def test_get_descendants_no_descendants(self):
        """Test the get_descendants() method for a record with no
        descendants."""
        with open(self.data_file('53658.html'), 'r') as fin:
            soup = BeautifulSoup(fin, 'lxml')
            self.assertEqual(get_descendants(soup), set([]))


if __name__ == '__main__':
    file_path = os.path.abspath(sys.argv[0])
    unittest.main()
else:
    file_path = os.path.abspath(sys.argv[0])



class Record:
    """
    Container class storing record of a mathematician in the graph.
    """

    def __init__(self, name, institution=None, year=None, id=None):
        """
        Record class constructor.

        Parameters:
            name: string containing mathematician's name
            institution: string containing mathematician's institution
                (empty if none)
            year: integer containing year degree was earned
            id: integer containing Math Genealogy Project id value
        """
        self.name = name
        self.institution = institution
        self.year = year
        self.id = id

        # Verify we got the types wanted.
        if not isinstance(self.name, basestring):
            raise TypeError(
                "Unexpected parameter type: expected string value \
for 'name'"
            )
        if (
            not isinstance(self.institution, basestring)
            and self.institution is not None
        ):
            raise TypeError(
                "Unexpected parameter type: expected string value \
for 'institution'"
            )
        if not isinstance(self.year, int) and self.year is not None:
            raise TypeError(
                "Unexpected parameter type: expected integer \
value for 'year'"
            )
        if not isinstance(self.id, int) and self.id is not None:
            raise TypeError(
                "Unexpected parameter type: expected integer \
value for 'id'"
            )

    def __cmp__(self, r2):
        """
        Compare a pair of mathematician records based on ids.
        """
        return self.id.__cmp__(r2.id)

    def __unicode__(self):
        if self.has_institution():
            if self.has_year():
                return u"{} \\n{} ({})".format(self.name, self.institution, self.year)
            else:
                return u"{} \\n{}".format(self.name, self.institution)
        else:
            if self.has_year():
                return u"{} \\n({})".format(self.name, self.year)
            else:
                return self.name

    def has_institution(self):
        """
        Return True if this record has an institution associated with it,
        else False.
        """
        return self.institution is not None

    def has_year(self):
        """
        Return True if this record has a year associated with it, else
        False.
        """
        return self.year is not None



import unittest
from geneagrapher.graph import Node, Record


class TestNodeMethods(unittest.TestCase):
    """Unit tests for the Node class."""
    def setUp(self):
        self.record = Record(u"Carl Friedrich Gau\xdf",
                             u"Universit\xe4t Helmstedt", 1799, 18231)

    def test_init(self):
        """Test the constructor."""
        node = Node(self.record, set(), set())
        self.assertEqual(node.record, self.record)
        self.assertEqual(node.ancestors, set())
        self.assertEqual(node.descendants, set())

    def test_init_bad_record(self):
        """Test the constructor for a case where the record passed is not a
        Record object."""
        self.assertRaises(TypeError, Node, 1, set(), set())

    def test_init_bad_ancestor_list(self):
        """Test the constructor for a case where the ancestor list is not a
        list."""
        self.assertRaises(TypeError, Node, self.record, 1, set())

    def test_2_init_bad_descendent_list(self):
        """Test the constructor for a case where the descendent list is not a
        list."""
        self.assertRaises(TypeError, Node, self.record, set(), 1)

    def test_unicode_full(self):
        """Test __unicode__() method for Node with complete record."""
        node = Node(self.record, set(), set())
        nodestr = node.__unicode__()
        nodestrexpt = u"Carl Friedrich Gau\xdf \\nUniversit\xe4t Helmstedt \
(1799)"
        self.assertEqual(nodestr, nodestrexpt)

    def test_unicode_no_year(self):
        """
        Test __unicode__() method for Node containing record without year.
        """
        record = Record(self.record.name, self.record.institution, None, 18231)
        node = Node(record, set(), set())
        nodestr = node.__unicode__()
        nodestrexpt = u"Carl Friedrich Gau\xdf \\nUniversit\xe4t Helmstedt"
        self.assertEqual(nodestr, nodestrexpt)

    def test_unicode_no_inst(self):
        """Test __unicode__() method for Node containing record without
        institution."""
        record = Record(self.record.name, None, 1799, 18231)
        node = Node(record, set(), set())
        nodestr = node.__unicode__()
        nodestrexpt = u"Carl Friedrich Gau\xdf \\n(1799)"
        self.assertEqual(nodestr, nodestrexpt)

    def test_unicode_no_inst_no_id(self):
        """Test __unicode__() method for Node containing record without
        institution or year."""
        record = Record(self.record.name, None, None, 18231)
        node = Node(record, set(), set())
        nodestr = node.__unicode__()
        nodestrexpt = u"Carl Friedrich Gau\xdf"
        self.assertEqual(nodestr, nodestrexpt)

    def test_cmp_equal(self):
        """Test comparison method for Nodes with identical records."""
        record2 = Record("Carl Friedrich Gauss", "Universitaet Helmstedt",
                         1799, 18231)
        node1 = Node(self.record, set(), set())
        node2 = Node(record2, set(), set())
        self.assert_(node1 == node2)

    def test_cmp_unequal(self):
        """Test comparison method for Nodes with different records."""
        record2 = Record("Leonhard Euler", "Universitaet Basel", 1726, 38586)
        node1 = Node(self.record, set(), set())
        node2 = Node(record2, set(), set())
        self.assert_(node1 < node2)

    def test_add_ancestor(self):
        """Test the add_ancestor() method."""
        node = Node(self.record, set(), set())
        node.add_ancestor(5)
        self.assertEqual(node.ancestors, set([5]))

    def test_add_ancestor_bad_type(self):
        """Test the add_ancestor() method for a case where the parameter type
        is incorrect."""
        node = Node(self.record, set(), set())
        self.assertRaises(TypeError, node.add_ancestor, '5')

    def test_get_id(self):
        """Test the get_id() method."""
        node = Node(self.record, set(), set())
        self.assertEqual(node.get_id(), 18231)

    def test_set_id(self):
        """Test the set_id() method."""
        node = Node(self.record, set(), set())
        self.assertEqual(node.get_id(), 18231)
        node.set_id(15)
        self.assertEqual(node.get_id(), 15)

if __name__ == '__main__':
    unittest.main()



from record import Record


class Node:
    """
    Container class storing a node in the graph.
    """

    def __init__(self, record, ancestors, descendants):
        """
        Node class constructor.

        Parameters:
            record: instance of the Record class
            ancestors: list of the record's genealogical ancestors'
                IDs
            descendants: list of this record's genealogical
                descendants' IDs
        """

        self.record = record
        self.ancestors = ancestors
        self.descendants = descendants

        # Verify parameter types.
        if not isinstance(self.record, Record):
            raise TypeError(
                "Unexpected parameter type: expected Record \
object for 'record'"
            )
        if not isinstance(self.ancestors, set):
            raise TypeError(
                "Unexpected parameter type: expected set object \
for 'ancestors'"
            )
        if not isinstance(self.descendants, set):
            raise TypeError(
                "Unexpected parameter type: expected set object \
for 'descendants'"
            )

    def __unicode__(self):
        return self.record.__unicode__()

    def __cmp__(self, n2):
        return self.record.__cmp__(n2.record)

    def __hash__(self):
        return hash(self.get_id())

    def add_ancestor(self, ancestor):  # NOTE: is this used?
        """
        Add an ancestor id to the ancestor set.
        """
        # Verify we were passed an int.
        if not isinstance(ancestor, int):
            raise TypeError(
                "Unexpected parameter type: expected int for \
'ancestor'"
            )
        self.ancestors.add(ancestor)

    def get_id(self):
        """
        Accessor method to retrieve the id of this node's record.
        """
        return self.record.id

    def set_id(self, id):  # NOTE: is this used?
        """
        Sets the record id.
        """
        self.record.id = id



import urllib
import re
from bs4 import BeautifulSoup


class Grabber:
    """
    Class for grabbing and parsing mathematician information from
    Math Genealogy Database.
    """

    def __init__(self, **kwargs):
        pass

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        pass

    def get_record(self, id):
        """
        For the mathematician in this object, extract the list of
        advisor ids, the mathematician name, the mathematician
        institution, and the year of the mathematician's degree.
        """
        url = "http://genealogy.math.ndsu.nodak.edu/id.php?id=" + str(id)
        page = urllib.urlopen(url)
        soup = BeautifulSoup(page, "lxml")
        page.close()

        return get_record_from_tree(soup, id)


def get_record_from_tree(soup, id):
    """Extract and return the fields in the mathematician record using the
    input tree."""
    if not has_record(soup):
        # Then a bad record id was given. Raise an exception.
        msg = "Invalid id {}".format(id)
        raise ValueError(msg)

    record = {}
    record["name"] = get_name(soup)
    record["institution"] = get_institution(soup)
    record["year"] = get_year(soup)
    record["advisors"] = get_advisors(soup)
    record["descendants"] = get_descendants(soup)
    return record


def has_record(soup):
    """Return True if the input tree contains a mathematician record and False
    otherwise."""
    if soup.p.string == "Non-numeric id supplied. Aborting.":
        # This is received, for instance, by going to
        # http://genealogy.math.ndsu.nodak.edu/id.php?id=999999999999999999999.
        return False
    return (
        not soup.p.string
        == u"You have specified an ID that does \
not exist in the database. Please back up and try again."
    )


def get_name(soup):
    """Extract the name from the given tree."""
    try:
        name = soup.find("h2").getText(strip=True)
    except AttributeError:
        name = ""
    return name


def get_institution(soup):
    """Return institution name (or None, if there is no institution name)."""
    try:
        institution = (
            soup.find(
                "div",
                style="line-height: 30px; \
text-align: center; margin-bottom: 1ex",
            )
            .find("span")
            .find("span")
            .text
        )
        if institution == u"":
            institution = None
    except AttributeError:
        institution = None
    return institution


def get_year(soup):
    """Return graduation year (or None, if there is no graduation year)."""
    try:
        inst_year = (
            soup.find(
                "div",
                style="line-height: 30px; text-align: \
center; margin-bottom: 1ex",
            )
            .find("span")
            .contents[-1]
            .strip()
        )
        if inst_year.isdigit():
            return int(inst_year)
        else:
            return None
    except AttributeError:
        return None


def get_advisors(soup):
    """Return the set of advisors."""
    return set(
        [
            extract_id(info.findNext())
            for info in soup.findAll(text=re.compile("Advisor"))
            if "Advisor: Unknown" not in info
        ]
    )


def get_descendants(soup):
    """Return the set of descendants."""
    if soup.find("table") is not None:
        return set([extract_id(info) for info in soup.find("table").findAll("a")])
    else:
        return set([])


def extract_id(tag):
    """Extract the ID from a tag with form <a href="id.php?id=7401">."""
    return int(tag.attrs["href"].split("=")[-1])



import re
import setuptools

with open("README.md", "r") as fin:
    long_description = fin.read()

long_description = re.sub(
    "^(!\[.*\]\()(.*\))",
    lambda m: m.group(1) + "https://github.com/davidalber/geneagrapher/raw/master/" + m.group(2),
    long_description,
    flags=re.MULTILINE
)

setuptools.setup(
    name="geneagrapher",
    version="1.0",
    author="David Alber",
    author_email="alber.david@gmail.com",
    description="Mathematical genealogy grapher.",
    entry_points={
        'console_scripts':
            ['ggrapher=geneagrapher.geneagrapher:ggrapher']
    },
    install_requires=['beautifulsoup4==4.6.3', 'lxml==4.2.5'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/davidalber/geneagrapher",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
